// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole.Util;

import java.util.*;
import java.text.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import com.ooc.CosNaming.OBNamingContextPackage.*;
import com.ooc.CosNamingConsole.GUI.*;
import com.ooc.CosNamingConsole.*;

final public class BindingUtil
{
    //
    // Error log object
    //
    private static MessageLogger logger_ =
        WindowFactory.getErrorWindowInstance().getLogger();

    //
    // Date formatter
    //
    private static java.text.DateFormat formatter_ =
        DateFormat.getDateTimeInstance();

    private BindingUtil()
    {
    }

    //
    // Convert time to local time
    //
    public static String
    getTime(long millis)
    {
	return formatter_.format(new Date(millis));
    }

    //
    // Get all bindings from a context
    //
    public static ExtendedBinding[]
    getBindingsFromContext(NamingContext nc)
    {
	ExtendedBinding[] bindings = new ExtendedBinding[0];

	//
	// Get bindings with timestamps from an extended context.
	// By a standard context only the standard binding is provided.
	//
	com.ooc.CosNaming.OBNamingContext obnc =
	    com.ooc.CosNaming.OBNamingContextHelper.narrow(nc);
	if(obnc != null)
	    bindings = obnc.list_extended();
	else
	{
	    BindingListHolder blh = new BindingListHolder();
	    BindingIteratorHolder bih = new BindingIteratorHolder();
	    nc.list(Integer.MAX_VALUE, blh, bih);
	    bindings = new ExtendedBinding[blh.value.length];

	    for(int i = 0 ; i < blh.value.length ; i++)
	    {
		org.omg.CORBA.Object obj =
		    BindingUtil.getObjectByName(nc, blh.value[i].binding_name);

		NcOrObj ncOrObj = new NcOrObj();
		if(blh.value[i].binding_type == BindingType.nobject)
		    ncOrObj.obj(obj);
		else
		{
		    NamingContext context = null;
		    try
		    {
			context = NamingContextHelper.narrow(obj);
		    }
		    catch(SystemException ex)
		    {
			// Ignore
		    }

		    if(context != null)
			ncOrObj.nc(context);
		    else
			ncOrObj.obj(obj);
		}

		bindings[i] = new ExtendedBinding(
		    blh.value[i].binding_name, ncOrObj, 0);
	    }

	    if(bih.value != null)
		bih.value.destroy();
	}

	return bindings;
    }

    //
    // Get all context bindings from a context
    //
    public static Vector
    getContextsFromContext(NamingContext nc)
    {
	ExtendedBinding[] bindings = getBindingsFromContext(nc);

	Vector contexts = new Vector(bindings.length);

	for(int i = 0 ; i < bindings.length ; i++)
	    if(bindings[i].ncOrObj.discriminator() == BindingType.ncontext)
		contexts.addElement(bindings[i]);

	contexts.trimToSize();

	return contexts;
    }

    //
    // Make deep copy of context
    //
    public static void
    deepCopy(ORB orb, NamingContext dst, NamingContext src)
	throws InvalidName, CannotProceed, NotFound, AlreadyBound
    {
	deepCopy(orb, dst, src, new Vector());
    }

    public static void
    deepCopy(ORB orb, NamingContext dst, NamingContext src, Vector contextList)
	throws InvalidName, CannotProceed, NotFound, AlreadyBound
    {
	String s = orb.object_to_string(src);

	//
	// Avoid recursion
	//
	if(contextList.contains(s))
	{
	    logger_.println(
		"Recursion detected, proceeding with next binding");
	    return;
	}

	contextList.addElement(s);

	BindingListHolder blh = new BindingListHolder();
	BindingIteratorHolder bih = new BindingIteratorHolder();
	src.list(Integer.MAX_VALUE, blh, bih);

	for(int i = 0 ; i < blh.value.length ; i++)
	{
	    Binding b = blh.value[i];

	    org.omg.CORBA.Object obj = src.resolve(b.binding_name);

	    if(b.binding_type == BindingType.ncontext)
	    {
		NamingContext newContext = dst.new_context();
		deepCopy(orb, newContext, NamingContextHelper.narrow(obj),
			 contextList);
		dst.bind_context(b.binding_name, newContext);
	    }
	    else
		dst.bind(b.binding_name, obj);
	}

	if(bih.value != null)
	    bih.value.destroy();
    }

    //
    // Recursively delete context contents
    //
    public static void
    deleteContents(ORB orb, NamingContext nc)
    {
	deleteContents(orb, nc, new Vector());
    }

    public static void
    deleteContents(ORB orb, NamingContext nc, Vector contextList)
    {
	String s = orb.object_to_string(nc);

	//
	// Avoid recursion
	//
	if(contextList.contains(s))
	{
	    logger_.println(
		"Recursion detected, proceeding with next binding");
	    return;
	}

	contextList.addElement(s);

	BindingListHolder blh = new BindingListHolder();
	BindingIteratorHolder bih = new BindingIteratorHolder();
	nc.list(Integer.MAX_VALUE, blh, bih);

	try
	{
	    for(int i = 0 ; i < blh.value.length ; i++)
	    {
		Binding b = blh.value[i];

		org.omg.CORBA.Object obj = nc.resolve(b.binding_name);

		if(b.binding_type == BindingType.ncontext)
		{
		    NamingContext target = NamingContextHelper.narrow(obj);
		    deleteContents(orb, target, contextList);
		    target.destroy();
		}

		nc.unbind(b.binding_name);
	    }
	}
	catch(SystemException ex)
	{
	    // Can be ignored
	}
	catch(UserException ex)
	{
	    // Can be ignored
	}

	if(bih.value != null)
	    bih.value.destroy();
    }

    //
    // Get context by its parent context and name
    //
    public static NamingContext
    getContextByName(NamingContext parent, NameComponent[] name)
    {
	NamingContext nc = null;

	try
	{
	    org.omg.CORBA.Object obj = parent.resolve(name);
	    nc = NamingContextHelper.narrow(obj);
	}
	catch(SystemException ex)
	{
	    logger_.printException(ex);
	}
	catch(UserException ex)
	{
	    logger_.printException(ex);
	}

	return nc;
    }

    //
    // Get object by its parent context and name
    //
    public static org.omg.CORBA.Object
    getObjectByName(NamingContext parent, NameComponent[] name)
    {
	org.omg.CORBA.Object obj = null;

	try
	{
	    obj = parent.resolve(name);
	}
	catch(SystemException ex)
	{
	    logger_.printException(ex);
	}
	catch(UserException ex)
	{
	    logger_.printException(ex);
	}

	return obj;
    }

    public static String
    getFullName(Binding b)
    {
	if(b.binding_name[0].kind.length() == 0)
	    return addEscapes(b.binding_name[0].id);
	else
	    return addEscapes(b.binding_name[0].id) + "." +
		addEscapes(b.binding_name[0].kind);
    }

    public static String
    getFullName(ExtendedBinding b)
    {
	if(b.binding_name[0].kind.length() == 0)
	    return addEscapes(b.binding_name[0].id);
	else
	    return addEscapes(b.binding_name[0].id) + "." +
		addEscapes(b.binding_name[0].kind);
    }

    public static String
    getFullName(NameComponent name)
    {
	if(name.kind.length() == 0)
	    return addEscapes(name.id);
	else
	    return addEscapes(name.id) + "." +
		addEscapes(name.kind);
    }

    private static String
    addEscapes(String str)
    {
	String result = new String();

	int len = str.length();
	for(int i = 0 ; i < len ; i++)
	{
	    char c = str.charAt(i);
	    switch(c)
	    {
	    case '.':
		result += "\\.";
		break;

	    case '/':
		result += "\\/";
		break;

	    case '\\':
		result += "\\\\";
		break;

	    default:
		result += c;
		break;
	    }
	}

	return result;
    }
}
