// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CORBA;

final class GIOPServerWorkerThreaded extends GIOPServerWorker
{
    private boolean destroy_; // True if destroy() was called
    private long connectionTimeout_; // The connection timeout
    private long timestamp_; // Timestamp for the last connection activity
    private ThreadPool threadPool_ = null; // The thread pool
    private ThreadGroup serverThreadGroup_; // The server thread group

    //
    // The sender thread
    //
    private final class SenderThread extends Thread
    {
        SenderThread(ThreadGroup group)
        {
            super(group, "ORBACUS:Server:SenderThread");
        }
        
	public void run()
	{
	    senderRun();

	    synchronized(GIOPServerWorkerThreaded.this)
	    {
		senderThread_ = null;
		GIOPServerWorkerThreaded.this.notifyAll();
	    }
	}
    };
    Thread senderThread_;

    //
    // The receiver thread
    //
    private final class ReceiverThread extends Thread
    {
        ReceiverThread(ThreadGroup group)
        {
            super(group, "ORBACUS:Server:ReceiverThread");
        }
        
	public void run()
	{
	    receiverRun();

	    synchronized(GIOPServerWorkerThreaded.this)
	    {
		receiverThread_ = null;
		GIOPServerWorkerThreaded.this.notifyAll();
	    }
	}
    };
    Thread receiverThread_;

    //
    // Monitor for "single threaded" thread model, which allows
    // only one thread for request execution.
    //
    private static Object singleThreadedMutex_ = new Object();

    //
    // Helper class for dispatching a request in case of a "thread per
    // request" concurrency model
    //
    private final class Dispatcher extends Thread
    {
	private InputStream in_;

	Dispatcher(ThreadGroup group, InputStream in)
	{
            super(group, "ORBACUS:ThreadPerRequest:Dispatcher");
	    in_ = in;
	}

	public void run()
	{
	    if(!incNumberOfRequests())
		return;
	    
	    try
	    {
		dispatchRequestSuper(in_);
	    }
	    finally
	    {
		decNumberOfRequests();
	    }
	}
    };

    //
    // Dispatch request helper
    //
    protected void
    dispatchRequestSuper(InputStream in)
    {
	super.dispatchRequest(in);
    }
    
    //
    // Update the timestamp
    //
    protected synchronized void
    updateTimestamp()
    {
	//
	// Only update if a connection timeout is set
	//
	if(connectionTimeout_ > 0)
	    timestamp_ = System.currentTimeMillis() / 1000;
    }

    //
    // Increment the number of active requests
    //
    protected synchronized boolean
    incNumberOfRequests()
    {
	if(destroy_)
	    return false;
	
	super.incNumberOfRequests();
	
	return true;
    }

    //
    // Decrement the number of active requests
    //
    protected synchronized void
    decNumberOfRequests()
    {
	super.decNumberOfRequests();
	
	if(numberOfRequests_ == 0)
	    notifyAll();
    }

    //
    // Add outgoing message
    //
    protected synchronized void
    add(com.ooc.OCI.Buffer buf)
    {
	if(!destroy_) // Don't add if already destroyed
	{
	    updateTimestamp();
	    super.add(buf);
	    
	    //
	    // Notify the sender thread. This is only necessary if
	    // this is the first outgoing message, since if there is
	    // more then one message in the buffers, threads are
	    // already notified.
	    //
	    if(outVec_.size() == 1)
		notifyAll();
	}
    }

    //
    // Remove outgoing message
    //
    protected synchronized com.ooc.OCI.Buffer
    remove()
    {
	//
	// Wait for a message
	//
	while(!destroy_ && outVec_.size() == 0)
	{
	    try
	    {
		if(connectionTimeout_ > 0)
		    wait(connectionTimeout_ * 1000);
		else
		    wait();
	    }
	    catch(InterruptedException ex)
	    {
	    }
	    
	    if(!destroy_ && outVec_.size() == 0 && connectionTimeout_ > 0 &&
	       numberOfRequests_ == 0)
	    {
		//
		// Get the current time
		//
		long current = System.currentTimeMillis() / 1000;
		
		//
		// Check whether the timeout limit has been exceeded
		//
		if(current >= timestamp_ + connectionTimeout_)
		{
		    //
		    // Destroy the worker
		    //
		    starter_.destroyWorker(this);
		}
	    }
	}
	
	//
	// Return nil buffer if destroy_ was set
	//
	if(destroy_)
	    return null;

	//
	// Remove and return first message from message buffers
	//
	if(outVec_.size() == 0)
	    throw new InternalError();

	com.ooc.OCI.Buffer buf =
	    (com.ooc.OCI.Buffer)outVec_.firstElement();

	outVec_.removeElementAt(0);

	return buf;
    }

    //
    // Dispatch a request
    //
    protected void
    dispatchRequest(InputStream in)
    {
	switch(BOA.conc_model().value())
	{
	case BOA.ConcModel._ConcModelBlocking:

	    throw new InternalError();
	    
	case BOA.ConcModel._ConcModelThreaded:

	    synchronized(singleThreadedMutex_)
	    {
		super.dispatchRequest(in);
	    }
	    break;
	
	case BOA.ConcModel._ConcModelThreadPerClient:
	    super.dispatchRequest(in);
	    break;
	    
	case BOA.ConcModel._ConcModelThreadPerRequest:
	{
	    new Dispatcher(serverThreadGroup_, in).start();
	    break;
	}
	 
	case BOA.ConcModel._ConcModelThreadPool:
	{
	    threadPool_.add(this, in);
	    break;
	}
	 
	default:
	    throw new InternalError();
	}
    }

    //
    // Constructor
    //
    GIOPServerWorkerThreaded(BOA boa, GIOPServerStarterThreaded starter,
			     com.ooc.OCI.Transport transport,
			     ThreadGroup serverThreadGroup,
			     ThreadPool threadPool)
    {
	super(boa, starter, transport);

	threadPool_ = threadPool;
	serverThreadGroup_ = serverThreadGroup;

	//
	// Cache properties
	//
	Properties properties = Properties.instance();
	String value;
	
	value = properties.getProperty("ooc.orb.server_timeout");
	if(value != null)
	{
	    connectionTimeout_ = Long.parseLong(value);
	    updateTimestamp();
	}
	else
	{
	    connectionTimeout_ = 0;
	    timestamp_ = 0;
	}

	//
	// Start sender and receiver thread
	//
	senderThread_ = new SenderThread(serverThreadGroup_);
	senderThread_.start();
	receiverThread_ = new ReceiverThread(serverThreadGroup_);
	receiverThread_.start();
    }

    //
    // Destructor
    //
    protected void
    finalize()
	throws Throwable
    {
	if(!destroy_)
	    throw new InternalError();

	super.finalize();
    }

    //
    // Destroy the worker
    //
    synchronized void
    _destroy()
    {
	//
	// Don't destroy twice
	//
	if(destroy_)
	    return;
	
	//
	// Add a CloseConnection message (must be done before destroy_
	// is set to true)
	//
	addCloseConnection();
	
	//
	// Send all remaining messages, ignoring any errors. This is
	// done only if this is the sender thread. Otherwise it's
	// possible that the sender thread interferes with sending the
	// remaining messages. Furthermore, it's not necessary to send
	// the remaining message here if this is not the sender
	// thread, because the sender thread will take care of this
	// before it exits.
	//
	if(senderThread_ == null ||
	   senderThread_.equals(Thread.currentThread()))
	{
	    sendRemainingMessages();
	}

	//
	// Set the destroy flag
	//
	destroy_ = true;
	
	//
	// Notify the sender and the receiver thread
	//
	notifyAll();
	
	//
	// Wait for the sender thread to terminate
	//
	if(senderThread_ != null &&
	   !senderThread_.equals(Thread.currentThread()))
	{
	    while(senderThread_ != null)
	    {
		try
		{
		    wait();
		}
		catch(InterruptedException ex)
		{
		}
	    }
	}
	
	//
	// Unblock threads blocking in the send() and recv()
	// operations
	//
	transport_.unblock_threads(false);
	
	//
	// Wait for the receiver thread to terminate
	//
	if(receiverThread_ != null &&
	   !receiverThread_.equals(Thread.currentThread()))
	{
	    while(receiverThread_ != null)
	    {
		try
		{
		    wait();
		}
		catch(InterruptedException ex)
		{
		}
	    }
	}
	
	//
	// Close the transport
	//
	closeTransport();
    }

    //
    // Run method for sender thread
    //
    public void
    senderRun()
    {
	while(!destroy_)
	{
	    try
	    {
		//
		// Remove outgoing message from the outgoing message buffers
		//
		com.ooc.OCI.Buffer buf = remove();
		
		//
		// Check if thread should be stopped and destroyed
		//
		if(buf == null)
		    break;
		
		//
		// Send buffer, blocking, detect connection loss
		//
		if(!transport_.send_detect(buf, true))
		{
		    //
		    // Destroy the worker
		    //
		    starter_.destroyWorker(this);

		    return; // Orderly shutdown
		}
		if(!buf.is_full())
		    throw new InternalError();
	    }
	    catch(org.omg.CORBA.SystemException ex)
	    {
		//
		// Destroy the worker
		//
		starter_.destroyWorker(this);

		return;
	    }
	}

	//
	// Send all remaining messages, ignoring any errors
	//
	sendRemainingMessages();
    }

    //
    // Run method for receiver thread
    //
    public void
    receiverRun()
    {
	while(!destroy_)
	{
	    try
	    {
		//
		// Setup the incoming message buffer
		//
		if(buf_ != null)
		    throw new InternalError();
		setupBuffer();
		
		//
		// Receive header, blocking, detect connection loss
		//
		if(!transport_.receive_detect(buf_, true))
		{
		    //
		    // Destroy the worker
		    //
		    starter_.destroyWorker(this);		
    
		    return; // Orderly shutdown
		}
		if(!buf_.is_full())
		    throw new InternalError();
		
		//
		// Check if thread should be stopped and destroyed
		//
		if(destroy_)
		    break;

		//
		// Header is complete
		//
		extractHeader();
		
		if(!buf_.is_full())
		{
		    //
		    // Receive body, blocking, detect connection loss
		    //
		    if(!transport_.receive_detect(buf_, true))
		    {
			//
			// Destroy the worker
			//
			starter_.destroyWorker(this);

			return; // Orderly shutdown
		    }
		    if(!buf_.is_full())
			throw new InternalError();
		}

		//
		// Execute the message
		//
		if(!destroy_)
		{
		    updateTimestamp();
		    execute();
		}
	    }
	    catch(org.omg.CORBA.SystemException ex)
	    {
		//
		// Destroy the worker
		//
		starter_.destroyWorker(this);

		return;
	    }
	}
    }
}
