// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package loan.dynamic;

public class DynamicServer
{
    private static final String company_ = "ACME Loan Corp";
    private static final String offerIdFile_ = "dynoffer.dat";

    // ------------------------------------------------------------------
    // main() and supporting methods
    // ------------------------------------------------------------------

    public static void
    main(String[] args)
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, props);
        org.omg.CORBA.BOA boa = ((com.ooc.CORBA.ORB)orb).BOA_init(args, props);

        //
        // Filter options
        //
        args = ((com.ooc.CORBA.ORB)orb).filter_options(args);
        args = ((com.ooc.CORBA.BOA)boa).filter_options(args);

        boolean showHelp = false, withdrawOffers = false;
        String productFile = null;

        //
        // Check arguments
        //
        for(int i = 0 ; i < args.length ; i++)
        {
            if(args[i].equals("-w"))
                withdrawOffers = true;
            else if(args[i].equals("-h"))
                showHelp = true;
            else if(productFile == null) // assume it is the product file
                productFile = args[i];
            else // unknown argument
                usage();
        }

        if(showHelp || productFile == null)
            usage();

        //
        // Initialize server event loop
        //
        ((com.ooc.CORBA.BOA)boa).init_servers();

        //
        // Connect to Trading Service
        //
        org.omg.CosTrading.Lookup trader = loan.util.DemoUtil.connect(orb);

        if(trader == null)
            usage();

        //
        // Clean up old offers
        //
        loan.util.DemoUtil.cleanOffers(trader, offerIdFile_);

        //
        // Quit now if -w option was specified
        //
        if(withdrawOffers)
            System.exit(0);

        //
        // Read in our products
        //
        java.util.Hashtable products = loadProducts(productFile);

        //
        // Instantiate our lender implementation
        //
        LenderImpl lender = new LenderImpl(company_, products);
        orb.connect(lender);

        //
        // Install the 'Mortgage' service type and export our service offers
        //
        loan.util.DemoUtil.installServiceType(orb, trader);
        exportOffers(orb, trader, products, lender);

        System.out.println("Server for '" + company_ + "' is ready...");
        boa.impl_is_ready(null);
    }

    static java.util.Hashtable
    loadProducts(String file)
    {
        java.util.Hashtable result = new java.util.Hashtable();

        //
        // Read our products from the products data file.  The file
        // is expected to have the following format:
        //
        // product_id,rate_min,rate_max,rate_inc,pct_down,years
        //
        // All blank lines and lines beginning with '#' are ignored.
        //

        try
        {
            System.out.println("Reading products from " + file + "...");

            java.io.FileReader fr = new java.io.FileReader(file);
            java.io.BufferedReader reader = new java.io.BufferedReader(fr);

            String line;
            while((line = reader.readLine()) != null)
            {
                String trimline = line.trim();

                //
                // Ignore blank lines and lines beginning with '#'
                //
                if(trimline.startsWith("#") || trimline.length() == 0)
                    continue;

                java.util.StringTokenizer tokenizer =
                    new java.util.StringTokenizer(trimline, ",");

                if(tokenizer.countTokens() != 6)
                {
                    System.err.println("Error:  Invalid line '" + line + 
                                       "' in file " + file);
                    System.exit(1);
                }

                ProductInfo p = new ProductInfo();
                String token;
                token = tokenizer.nextToken();
                p.productId = token;
                token = tokenizer.nextToken();
                p.rateMin = Float.valueOf(token).floatValue();
                token = tokenizer.nextToken();
                p.rateMax = Float.valueOf(token).floatValue();
                token = tokenizer.nextToken();
                p.rateIncrement = Float.valueOf(token).floatValue();
                token = tokenizer.nextToken();
                p.pctDown = Float.valueOf(token).floatValue();
                token = tokenizer.nextToken();
                p.years = Short.valueOf(token).shortValue();

                result.put(p.productId, p);
            }

            fr.close();
        }
        catch(java.io.FileNotFoundException ex)
        {
            System.err.println("Error:  Unable to open file " + file);
            System.exit(1);
        }
        catch(java.io.IOException ex)
        {
            System.err.println("Error:  Unable to read file " + file);
            System.exit(1);
        }
        catch(NumberFormatException ex)
        {
            System.err.println("Error:  Invalid number in file " +
                               file + ": " + ex.getMessage());
            System.exit(1);
        }

        return result;
    }

    static void
    exportOffers(org.omg.CORBA.ORB orb,
                 org.omg.CosTrading.Lookup trader,
                 java.util.Hashtable products,
                 loan.LenderDemo.Lender lender)
    {
        String result = null;

        System.out.println("Exporting service offers...");

        try
        {
            //
            // Write the ID of each offer we export to a file
            //
            java.io.FileWriter fw = new java.io.FileWriter(offerIdFile_);
            java.io.PrintWriter writer = new java.io.PrintWriter(fw);

            org.omg.CosTrading.Register reg = trader.register_if();

            if(reg == null)
            {
                System.err.println("Error:  Trader does not support " +
                                   "the Register interface");
                System.exit(1);
            }

            java.util.Enumeration elements = products.elements();
            while(elements.hasMoreElements())
            {
                ProductInfo prod = (ProductInfo) elements.nextElement();

                org.omg.CosTrading.Property[] props =
                    new org.omg.CosTrading.Property[4];

                props[0] = new org.omg.CosTrading.Property();
                props[0].name = "product_id";
                props[0].value = orb.create_any();
                props[0].value.insert_string(prod.productId);

                //
                // Create dynamic property - the RateEval object will
                //     provide the actual value for the rate property
                //
                props[1] = new org.omg.CosTrading.Property();
                props[1].name = "rate";
                props[1].value = orb.create_any();
                org.omg.CosTradingDynamic.DynamicProp dp =
                    new org.omg.CosTradingDynamic.DynamicProp();
                dp.eval_if = new RateEval(orb, prod);
                orb.connect(dp.eval_if);
                dp.returned_type = orb.get_primitive_tc(
                    org.omg.CORBA.TCKind.tk_float);
                dp.extra_info = orb.create_any();
                org.omg.CosTradingDynamic.DynamicPropHelper.insert(
                    props[1].value, dp);

                props[2] = new org.omg.CosTrading.Property();
                props[2].name = "pct_down";
                props[2].value = orb.create_any();
                props[2].value.insert_float(prod.pctDown);

                props[3] = new org.omg.CosTrading.Property();
                props[3].name = "years";
                props[3].value = orb.create_any();
                props[3].value.insert_short(prod.years);

                String id = reg.export(lender,
                                       loan.util.DemoUtil.getServiceType(),
                                       props);
                writer.println(id);
            }

            writer.flush();
            fw.close();
        }
        catch(java.io.IOException ex)
        {
            System.err.println("Error:  Unable to write file " + offerIdFile_);
            System.exit(1);
        }
        catch(org.omg.CosTrading.RegisterPackage.InvalidObjectRef e)
        {
            System.err.println("Error:  Invalid object reference");
            System.exit(1);
        }
        catch(org.omg.CosTrading.IllegalServiceType e)
        {
            System.err.println("Error:  Illegal service type '" + e.type +
                               "'");
            System.exit(1);
        }
        catch(org.omg.CosTrading.UnknownServiceType e)
        {
            System.err.println("Error:  Unknown service type '" + e.type +
                               "'");
            System.exit(1);
        }
        catch(org.omg.CosTrading.RegisterPackage.InterfaceTypeMismatch e)
        {
            System.err.println("Error:  Interface type mismatch for '" +
                               e.type + "'");
            System.exit(1);
        }
        catch(org.omg.CosTrading.IllegalPropertyName e)
        {
            System.err.println("Error:  Illegal property name '" +
                               e.name + "'");
            System.exit(1);
        }
        catch(org.omg.CosTrading.PropertyTypeMismatch e)
        {
            System.err.println("Error:  Property type mismatch for '" +
                               e.prop.name + "'");
            System.exit(1);
        }
        catch(org.omg.CosTrading.ReadonlyDynamicProperty e)
        {
            System.err.println("Error:  Readonly dynamic property '" +
                               e.name + "'");
            System.exit(1);
        }
        catch(org.omg.CosTrading.MissingMandatoryProperty e)
        {
            System.err.println("Error:  Missing mandatory property '" +
                               e.name + "'");
            System.exit(1);
        }
        catch(org.omg.CosTrading.DuplicatePropertyName e)
        {
            System.err.println("Error:  Duplicate property '" +
                               e.name + "'");
            System.exit(1);
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("System error occurred");
            e.printStackTrace();
            System.exit(1);
        }
    }

    static void
    usage()
    {
        System.err.println();
        System.err.println("Usage:  java loan.dynamic.DynamicServer [-h] " +
                           "[-w] product-file");
        System.err.println();
        System.err.println("Options:");
        System.err.println();
        System.err.println("  -h   Show this help message");
        System.err.println("  -w   Withdraws existing offers and exits");
        System.exit(1);
    }
}
