// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// This class is responsible for evaluating all "local" service offers.
// Two thread pools are used to feed service offers requiring special handling
// (i.e. offers containing dynamic properties and proxy offers) to
// independent threads for simultaneous evaluation.
//
// The final result is a Vector of PropertyList objects that have
// matched the constraint.
//
final class OfferEvaluator
{
    private org.omg.CORBA.ORB orb_;
    private String type_;
    private String constraint_;
    private Constraint constraintObj_;
    private String preference_;
    private org.omg.CosTrading.Policy[] policies_;
    private org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps_;
    private java.util.Vector limits_;
    private java.util.Vector propertyLists_;
    private int matchCard_;
    private int matchCount_;
    private java.util.Vector results_;
    private int numProcessed_;
    private int threadPriority_;
    private int numDynamicThreads_;
    private int numProxyThreads_;
    private ThreadPool proxyPool_;
    private ThreadPool dynamicPool_;

    OfferEvaluator(org.omg.CORBA.ORB orb,
                   String type,
                   String constraint,
                   Constraint constraintObj,
                   String preference,
                   org.omg.CosTrading.Policy[] policies,
                   org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps,
                   java.util.Vector limits,
                   java.util.Vector propertyLists,
                   int matchCard)
    {
        //
        // Save arguments
        //
        orb_               = orb;
        type_              = type;
        constraint_        = constraint;
        constraintObj_     = constraintObj;
        preference_        = preference;
        policies_          = policies;
        desiredProps_      = desiredProps;
        limits_            = limits;
        propertyLists_     = propertyLists;
        matchCard_         = matchCard;

        matchCount_        = 0;
        results_           = new java.util.Vector();
        numProcessed_      = 0;

        //
        // Setup the ThreadPool for evaluating proxy offers
        //
        ProxyEval p = new ProxyEval(orb_, type_, constraint_, constraintObj_,
                                    preference_, policies_, desiredProps_,
                                    limits_);
        int maxProxy = 10; // TODO: need a property for this?
        proxyPool_ = new ThreadPool(p, maxProxy);

        //
        // Setup the ThreadPool for evaluating offers with dynamic properties
        //
        DynEval d = new DynEval(constraintObj_, desiredProps_);
        int maxDynamic =  10; // TODO: need a property for this?
        dynamicPool_ = new ThreadPool(d, maxDynamic);
    }

    //
    // Process the offers; we iterate over all of the given results,
    // dispatching them to the appropriate thread pools or evaluating the
    // simple offers immediately.
    //
    // Offers with dynamic properties and proxy offers are handled in separate
    // threads.  We give a higher priority to these threads; if we didn't, the
    // loop in this method might starve the threads that are evaluating offers,
    // forcing evaluation to occur after all other offers.  We at least want to
    // give the threads a chance of being included, if our match cardinality
    // is limited.
    //
    public synchronized java.util.Vector
    getResults()
    {
        java.util.Enumeration e = propertyLists_.elements();
        while(e.hasMoreElements() && !getDone())
        {
            PropertyList pl = (PropertyList)e.nextElement();

            if(pl instanceof ProxyPropertyList)
                proxyPool_.schedule(pl);
            else if(PropUtil.hasDynamicProperties(pl.getProperties()))
                dynamicPool_.schedule(pl);
            else
            {
                if(constraintObj_.evaluate(pl))
                    offerNotify(pl);
                resultNotify(pl);
            }
        }

        //
        // Wait until we're done, and then deactivate the queues
        //
        while(!getDone())
        {
            try
            {
                wait();  // Wait until we are notified and check again
            }
            catch(InterruptedException ex)
            {
            }
        }

        //
        // Halting the pools signals the threads to terminate
        //
        dynamicPool_.halt();
        proxyPool_.halt();
        dynamicPool_.finish();
        proxyPool_.finish();

        return results_;
    }

    public synchronized boolean
    getDone()
    {
        //
        // We're done when we've considered all the offers or
        // when matchCount_ == matchCard_
        //
        boolean result = (matchCard_ == matchCount_ ||
                          numProcessed_ == propertyLists_.size());

        return result;
    }

    //
    // Called when an offer has been evaluated
    //
    protected synchronized boolean
    offerNotify(PropertyList pl)
    {
        //
        // Add property list to the results
        //
        addResult(pl);

        boolean done = getDone();
        if(done)
            notifyAll();

        //
        // Return true if we can accept more offers
        //
        return (!done);
    }

    //
    // Called when a result has been processed and return true if
    // we are done
    //
    protected synchronized boolean
    resultNotify(PropertyList pl)
    {
        //
        // We keep track of the number of original PropertyList objects
        // we've processed, so that we know when we've evaluated all of
        // them; a single PropertyList object may result in numerous
        // calls to offerNotify (for proxy offers)
        //
        numProcessed_++;

        boolean done = getDone();
        if(done)
            notifyAll();

        return done;
    }

    //
    // Adds another match to our list of results
    //
    protected synchronized void
    addResult(PropertyList pl)
    {
        //
        // We may be restricted by matchCard_, therefore we check
        // to make sure we're not done before adding more matching
        // offers
        //
        if(matchCount_ < matchCard_)
        {
            results_.addElement(pl);
            matchCount_++;
        }
    }



    //
    // DynEval is responsible for evaluating offers with dynamic properties
    //
    protected class DynEval implements ThreadPool.Worker
    {
        private Constraint constraintObj_;
        private org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps_;


        public
        DynEval(Constraint constraint,
                org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps)
        {
            constraintObj_ = constraint;
            desiredProps_ = desiredProps;
        }

        //
        // Evaluate an offer with dynamic properties and return true if
        // the ThreadPool should continue, false otherwise
        //
        public boolean
        doWork(Object o)
        {
            PropertyList pl = (PropertyList)o;

            //
            // First evaluate the constraint with this source
            //

            if(constraintObj_.evaluate(pl))
            {
                org.omg.CosTrading.Property[] props = pl.getProperties();

                //
                // The constraint succeeded, now we need to ensure that
                // the PropertyList has evaluated all of the "desired"
                // properties
                //

                if(desiredProps_.discriminator() ==
                   org.omg.CosTrading.LookupPackage.HowManyProps.all)
                {
                    for(int i = 0 ; i < props.length ; i++)
                        pl.getPropertyValue(props[i].name);
                }
                else if(desiredProps_.discriminator() ==
                   org.omg.CosTrading.LookupPackage.HowManyProps.some)
                {
                    String[] names = desiredProps_.prop_names();
                    for(int i = 0 ; i < names.length ; i++)
                        pl.getPropertyValue(names[i]);
                }

                //
                // Inform our OfferEvaluator owner that a matching
                // offer has been found
                //
                OfferEvaluator.this.offerNotify(pl);
            }

            //
            // Inform our OfferEvaluator owner that we've processed a source.
            // Return true if the ThreadPool should continue, false otherwise.
            //
            return !OfferEvaluator.this.resultNotify(pl);
        }
    }



    //
    // ProxyEval is responsible for evaluating proxy offers
    //
    protected class ProxyEval implements ThreadPool.Worker
    {
        private org.omg.CORBA.ORB orb_;
        private String type_;
        private String constraint_;
        private Constraint constraintObj_;
        private String preference_;
        private org.omg.CosTrading.Policy[] policies_;
        private org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps_;
        private java.util.Vector limits_;


        public
        ProxyEval(org.omg.CORBA.ORB orb,
                  String type,
                  String constraint,
                  Constraint constraintObj,
                  String preference,
                  org.omg.CosTrading.Policy[] policies,
                  org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps,
                  java.util.Vector limits)
        {
            orb_           = orb;
            type_          = type;
            constraint_    = constraint;
            constraintObj_ = constraintObj;
            preference_    = preference;
            policies_      = policies;
            desiredProps_  = desiredProps;
            limits_        = limits;
        }

        //
        // Return true if the ThreadPool should continue, false otherwise
        //
        public boolean
        doWork(Object o)
        {
            ProxyPropertyList pl = (ProxyPropertyList)o;

            org.omg.CosTrading.ProxyPackage.ProxyInfo info = pl.getInfo();

            boolean match = false;

            //
            // If if_match_all is true, then type conformance is all that
            // is required for this proxy offer to be considered a match
            //
            if(info.if_match_all)
                match = true;
            else  // Otherwise evaluate the offer against the constraint
                match = constraintObj_.evaluate(pl);

            if(match)
            {
                //
                // Rewrite the constraint; this may fail if the constraint
                // uses a dynamic property for which a value could not be
                // obtained
                //
                String constraint =
                    Recipe.rewrite(info.recipe, pl, constraint_);

                if(constraint != null)
                    evalOffer(pl, info, constraint);
            }

            //
            // Inform our OfferEvaluator owner that we've processed a result.
            // Return true if the ThreadPool should continue, false otherwise.
            //
            return !OfferEvaluator.this.resultNotify(pl);
        }

        protected void
        evalOffer(ProxyPropertyList pl,
                  org.omg.CosTrading.ProxyPackage.ProxyInfo info,
                  String newConstraint)
        {
            //
            // Build a new array of policies, appending the
            // "policies to pass on"
            //
            int numPolicies =
                policies_.length + info.policies_to_pass_on.length;
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[numPolicies];

            int count = 0;
            while(count < policies_.length)
            {
                policies[count] = policies_[count];
                count++;
            }

            for(int i = 0 ; i < info.policies_to_pass_on.length ; i++)
                policies[count++] = info.policies_to_pass_on[i];

            try
            {
                org.omg.CosTrading.OfferSeqHolder offers =
                    new org.omg.CosTrading.OfferSeqHolder();
                org.omg.CosTrading.OfferIteratorHolder iter =
                    new org.omg.CosTrading.OfferIteratorHolder();
                org.omg.CosTrading.PolicyNameSeqHolder limits =
                    new org.omg.CosTrading.PolicyNameSeqHolder();

                //
                // Call the target trader
                //
                info.target.query(type_, newConstraint, preference_,
                                  policies, desiredProps_, 0,
                                  offers, iter, limits);

                //
                // Concatenate limits (if any)
                //
                if(limits.value != null && limits.value.length > 0)
                {
                    synchronized(limits_)
                    {
                        for(int i = 0 ; i < limits.value.length ; i++)
                            if(!limits_.contains(limits.value[i]))
                                limits_.addElement(limits.value[i]);
                    }
                }

                //
                // Process any offers we received
                //
                if(iter.value != null)
                {
                    org.omg.CosTrading.OfferSeqHolder seq =
                        new org.omg.CosTrading.OfferSeqHolder();

                    boolean more;
                    do
                    {
                        more = iter.value.next_n(20, seq);
                        for(int i = 0 ; i < seq.value.length ; i++)
                        {
                            org.omg.CosTrading.Offer o = seq.value[i];
                            //
                            // Create a new PropertyList object
                            // for each offer
                            //
                            PropertyList list =
                                new PropertyList(orb_, o.reference,
                                                 o.properties);

                            //
                            // Inform our OfferEvaluator owner that a matching
                            // offer has been found
                            //
                            if(!OfferEvaluator.this.offerNotify(list))
                                break;
                        }
                    }
                    while(more);

                    iter.value.destroy();
                }
            }
            catch(org.omg.CORBA.UserException e)
            {
                // ignore
            }
            catch(org.omg.CORBA.SystemException e)
            {
                // ignore
            }
        }
    }
}
