// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Used for validating and rewriting proxy constraint recipes
//
final class Recipe
{
    private static final String languageID_ = "<<OMG 1.0>>";

    //
    // Validates a proxy recipe; requires properties mentioned in the
    // recipe to be present in props
    //
    static boolean
    validate(String recipe, org.omg.CosTrading.Property[] props)
    {
        recipe = checkLanguage(recipe);
        if(recipe == null)
            return false;

        boolean result = true;

        int pos = 0;
        int len = recipe.length();

        //
        // Make a hashtable of properties for quicker lookup
        //
        java.util.Hashtable propTable = new java.util.Hashtable();
        for(int i = 0 ; i < props.length ; i++)
            propTable.put(props[i].name, props[i].value);

        boolean seenDollar = false;

        while(pos < len && result)
        {
            char ch = recipe.charAt(pos);

            switch(ch)
            {
                case '$':
                    if(seenDollar)
                        seenDollar = false;
                    else
                        seenDollar = true;
                    break;

                case '*':
                    if(seenDollar)
                        seenDollar = false;
                    break;

                case '(':
                    if(seenDollar)
                    {
                        int rparen = recipe.indexOf(')', pos);
                        if(rparen < 0)
                            result = false;  // missing right paren
                        else
                        {
                            //
                            // Extract the property name
                            //
                            String propName =
                                recipe.substring(pos + 1, rparen);

                            //
                            // Make sure the property exists
                            //
                            if(! propTable.containsKey(propName))
                                result = false;
                            else
                            {
                                org.omg.CORBA.Any any =
                                    (org.omg.CORBA.Any)propTable.get(propName);
                                org.omg.CORBA.TypeCode tc = any.type();
                                if(PropUtil.isDynamicProperty(tc))
                                {
                                    org.omg.CosTradingDynamic.DynamicProp dp =
                                        org.omg.CosTradingDynamic.
                                        DynamicPropHelper.extract(any);
                                    tc = dp.returned_type;
                                }

                                result = checkPropertyType(tc);
                            }

                            pos = rparen;
                        }

                        seenDollar = false;
                    }
                    break;
            }  // switch(ch)

            pos++;
        }  // while

        if(seenDollar)
            result = false;

        return result;
    }

    //
    // Returns the secondary constraint by following the recipe; assumes
    // the recipe has previously been validated; returns null if a value
    // for a required property could not be obtained
    //
    static String
    rewrite(String recipe, PropertyList pl, String primary)
    {
        recipe = checkLanguage(recipe);
        if(recipe == null)
            return null;

        StringBuffer result = new StringBuffer();

        boolean seenDollar = false;
        int pos = 0;
        int len = recipe.length();

        while(pos < len)
        {
            char ch = recipe.charAt(pos);

            switch(ch)
            {
                case '$':
                    if(seenDollar)
                    {
                        result.append('$');
                        seenDollar = false;
                    }
                    else
                        seenDollar = true;
                    break;

                case '*':
                    if(seenDollar)
                    {
                        seenDollar = false;
                        result.append(primary);
                    }
                    else
                        result.append(ch);
                    break;

                case '(':
                    if(seenDollar)
                    {
                        int rparen = recipe.indexOf(')', pos);
                        //
                        // Extract the property name
                        //
                        String propName = recipe.substring(pos + 1, rparen);
                        //
                        // Use the property list to get the property's value,
                        // which automatically handles dynamic properties
                        //
                        org.omg.CORBA.Any value =
                            pl.getPropertyValue(propName);
                        if(value == null)
                            return null;
                        result.append(convertValue(value));
                        pos = rparen;
                        seenDollar = false;
                    }
                    else
                        result.append(ch);
                    break;

                default:
                    result.append(ch);
            }  // switch(ch)

            pos++;
        }  // while

        return result.toString();
    }

    //
    // Determines if the given property type is allowable in a recipe
    //
    protected static boolean
    checkPropertyType(org.omg.CORBA.TypeCode tc)
    {
        boolean result = false;

        org.omg.CORBA.TCKind kind = tc.kind();
        switch(kind.value())
        {
            //
            // These are the types of properties supported in a recipe
            //
            case org.omg.CORBA.TCKind._tk_short:
            case org.omg.CORBA.TCKind._tk_long:
            case org.omg.CORBA.TCKind._tk_ushort:
            case org.omg.CORBA.TCKind._tk_ulong:
            case org.omg.CORBA.TCKind._tk_float:
            case org.omg.CORBA.TCKind._tk_double:
            case org.omg.CORBA.TCKind._tk_boolean:
            case org.omg.CORBA.TCKind._tk_char:
            case org.omg.CORBA.TCKind._tk_string:
            case org.omg.CORBA.TCKind._tk_octet:
                result = true;
                break;
        }

        return result;
    }

    //
    // Converts a property value to a string
    //
    protected static String
    convertValue(org.omg.CORBA.Any val)
    {
        String result = "<unknown>";

        org.omg.CORBA.TCKind kind = val.type().kind();

        try
        {
            switch(kind.value())
            {
                case org.omg.CORBA.TCKind._tk_short:
                {
                    int s = val.extract_short();
                    result = "" + s;
                    break;
                }

                case org.omg.CORBA.TCKind._tk_long:
                {
                    int l = val.extract_long();
                    result = "" + l;
                    break;
                }

                case org.omg.CORBA.TCKind._tk_ushort:
                {
                    int i = val.extract_ushort();
                    if(i < 0)
                        i += ((int)Short.MAX_VALUE - (int)Short.MIN_VALUE) + 1;
                    result = "" + i;
                    break;
                }

                case org.omg.CORBA.TCKind._tk_ulong:
                {
                    long l = val.extract_ulong();
                    if(l < 0)
                        l += ((long)Integer.MAX_VALUE -
                              (long)Integer.MIN_VALUE) + 1;
                    result = "" + l;
                    break;
                }

                case org.omg.CORBA.TCKind._tk_float:
                {
                    float f = val.extract_float();
                    result = "" + f;
                    break;
                }

                case org.omg.CORBA.TCKind._tk_double:
                {
                    double d = val.extract_double();
                    result = "" + d;
                    break;
                }

                case org.omg.CORBA.TCKind._tk_boolean:
                {
                    boolean b = val.extract_boolean();
                    result = "" + (b ? "TRUE" : "FALSE");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_char:
                {
                    char c = val.extract_char();
                    result = "'" + c + "'";
                    break;
                }

                case org.omg.CORBA.TCKind._tk_string:
                {
                    String s = val.extract_string();
                    result = "'" + s + "'";
                    break;
                }

                case org.omg.CORBA.TCKind._tk_octet:
                {
                    short s = val.extract_octet();
                    if(s < 0)
                        s += ((short)Byte.MAX_VALUE -
                              (short)Byte.MIN_VALUE) + 1;
                    result = "" + s;
                    break;
                }
            }
        }
        catch(org.omg.CORBA.BAD_OPERATION e)
        {
            e.printStackTrace();
        }

        return result;
    }

    //
    // Validate and remove the language identifier from the recipe, if present
    //
    protected static String
    checkLanguage(String recipe)
    {
        String result = recipe;

        if(result.startsWith("<<"))
        {
            if(result.startsWith(languageID_))
                result = result.substring(languageID_.length());
            else
                result = null;
        }

        return result;
    }
}
