// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

final class Scanner
{
    private String input_;
    private int pos_;
    private int token_;
    private String lexeme_;
    private boolean eof_;

    private static String[] keywords_ =
    {
        "min",
        "max",
        "with",
        "random",
        "first",
        "and",
        "or",
        "not",
        "in",
        "exist",
        "TRUE",
        "FALSE"
    };

    Scanner(String input)
    {
        input_ = input;
        pos_ = 0;
        token_ = Tokens.TOK_END;
        eof_ = false;

        //
        // Check for presence of <<OMG 1.0>> language identifier
        //
        if(input_.startsWith("<<"))
        {
            if(input_.startsWith("<<OMG 1.0>>"))
            {
                pos_ = 11;
                next();
            }
            else
                token_ = Tokens.TOK_ERROR;
        }
        else
            next();
    }

    private boolean
    eof()
    {
        return eof_;
    }

    private char
    nextChar()
    {
        char result = 0;

        if(pos_ < input_.length())
        {
            result = input_.charAt(pos_);
            pos_++;
        }
        else
            eof_ = true;

        return result;
    }

    private void
    clearLexeme()
    {
        lexeme_ = "";
    }

    private void
    addLexeme(char c)
    {
        lexeme_ += c;
    }

    private boolean
    parseIdent(char c)
    {
        boolean result;

        //
        // Allow backslash as first character of identifier
        //
        if(c == '\\')
        {
            addLexeme(c);
            c = nextChar();
        }

        if(c != '_' && !Character.isLetter(c))
        {
            result = false;
        }
        else
        {
            while(!eof() && (c == '_' || Character.isLetterOrDigit(c)))
            {
                addLexeme(c);
                c = nextChar();
            }

            if(!eof())
                pushBack();

            result = true;
        }

        return result;
    }

    int
    token()
    {
        return token_;
    }

    int
    next()
    {
        int result = Tokens.TOK_ERROR;

        boolean done = false;

        clearLexeme();

        while(!done)
        {
            char c = nextChar();

            if(eof())
            {
                result = Tokens.TOK_END;
                break;
            }

            switch(c)
            {
            case '(':
                result = Tokens.TOK_LPAREN;
                done = true;
                addLexeme(c);
                break;

            case ')':
                result = Tokens.TOK_RPAREN;
                done = true;
                addLexeme(c);
                break;

            case '[':
                result = Tokens.TOK_LBRACKET;
                done = true;
                addLexeme(c);
                break;

            case ']':
                result = Tokens.TOK_RBRACKET;
                done = true;
                addLexeme(c);
                break;

            case '-':
                result = Tokens.TOK_MINUS;
                done = true;
                addLexeme(c);
                break;

            case '+':
                result = Tokens.TOK_PLUS;
                done = true;
                addLexeme(c);
                break;

            case '*':
                result = Tokens.TOK_MULT;
                done = true;
                addLexeme(c);
                break;

            case '/':
                result = Tokens.TOK_DIV;
                done = true;
                addLexeme(c);
                break;

            case '~':
                result = Tokens.TOK_TWIDDLE;
                done = true;
                addLexeme(c);
                break;

            case '=':
            {
                char peek = nextChar();
                if(peek == '=')
                {
                    result = Tokens.TOK_EQ;
                    addLexeme(c);
                    addLexeme(peek);
                }
                else
                {
                    result = Tokens.TOK_ERROR;
                    pushBack();
                }
                done = true;
                break;
            }

            case '!':
            {
                char peek = nextChar();
                if(peek == '=')
                {
                    result = Tokens.TOK_NE;
                    addLexeme(c);
                    addLexeme(peek);
                }
                else
                {
                    result = Tokens.TOK_ERROR;
                    pushBack();
                }
                done = true;
                break;
            }

            case '<':
            {
                addLexeme(c);
                char peek = nextChar();

                if(peek == '=')
                {
                    result = Tokens.TOK_LE;
                    addLexeme(peek);
                }
                else
                {
                    result = Tokens.TOK_LT;
                    pushBack();
                }
                done = true;
                break;
            }

            case '>':
            {
                addLexeme(c);
                char peek = nextChar();

                if(peek == '=')
                {
                    result = Tokens.TOK_GE;
                    addLexeme(peek);
                }
                else
                {
                    result = Tokens.TOK_GT;
                    pushBack();
                }
                done = true;
                break;
            }

            case '\'':
            {
                result = Tokens.TOK_STRING;
                while((c = nextChar()) != '\'')
                {
                    if(c == '\\')
                        c = nextChar();

                    if(eof())
                    {
                        result = Tokens.TOK_ERROR;
                        break;
                    }

                    addLexeme(c);
                }
                done = true;
                break;
            }

            case '.':
                result = Tokens.TOK_DOT;
                done = true;
                addLexeme(c);
                break;

            case '0':
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
            {
                while(Character.isDigit(c))
                {
                    addLexeme(c);
                    c = nextChar();
                }

                if(!eof())
                    pushBack();

                result = Tokens.TOK_DIGITS;
                done = true;
                break;
            }

            case ' ':
            case '\t':
                continue;
            }

            if(!done)
            {
                if(parseIdent(c))
                {
                    String lexeme;
                    if(lexeme_.charAt(0) == '\\')
                        lexeme = lexeme_.substring(1);
                    else
                        lexeme = lexeme_;

                    //
                    // See if lexeme matches a keyword
                    //
                    result = Tokens.TOK_IDENT;
                    for(int n = 0 ; n < keywords_.length ; n++)
                    {
                        if(lexeme.equals(keywords_[n]))
                        {
                            result = Tokens.TOK_KEYWORD;
                            break;
                        }
                    }
                }

                done = true;
            }
        }

        token_ = result;
        return result;
    }

    void
    pushBack()
    {
        if(pos_ <= 0)
            throw new RuntimeException();
        pos_--;
    }

    String
    lexeme()
    {
        return lexeme_;
    }
}
