// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// ThreadPool uses a collection of "worker" threads to simultaneously
// perform a task on multiple objects
//
final class ThreadPool implements Runnable
{
    private Worker worker_;
    private int maxThreads_;
    private Thread[] threads_;
    private int threadCount_;
    private int threadPriority_;
    private com.ooc.Util.MessageQueue queue_;


    //
    // The client of a ThreadPool must implement this interface.
    // The doWork() method is invoked once for each object that
    // is scheduled by the pool.
    //
    public interface Worker
    {
        //
        // Return true if the ThreadPool should continue, false otherwise
        //
        public boolean
        doWork(Object o);
    }


    public
    ThreadPool(Worker worker, int maxThreads)
    {
        this(worker, maxThreads, Thread.currentThread().getPriority() + 1);
    }

    public
    ThreadPool(Worker worker, int maxThreads, int priority)
    {
        worker_ = worker;
        maxThreads_ = maxThreads;
        threadPriority_ = priority;
        threads_ = new Thread[maxThreads_];
        threadCount_ = 0;
        queue_ = new com.ooc.Util.MessageQueue();
    }

    //
    // Schedule a worker to handle this object
    //
    public synchronized void
    schedule(Object o)
    {
        //
        // if we haven't exceeded our thread limit, create a new thread
        //
        if(threadCount_ < maxThreads_)
        {
            Thread t = new Thread(this);
            threads_[threadCount_++] = t;
            t.setPriority(threadPriority_);
            t.start();
        }

        //
        // insert the object into the message queue; the next
        // available thread will retrieve it
        //
        queue_.enqueue(o);
    }

    //
    // Wait until all worker threads have terminated; the worker threads
    // must terminate on their own, or halt() must be invoked, before
    // this method will return
    //
    public void
    finish()
    {
        waitThreads();
    }

    //
    // Terminate worker threads
    //
    public void
    halt()
    {
        // deactivating the queue will eventually terminate all threads
        queue_.deactivate();
    }

    //
    // Join all worker threads
    //
    protected void
    waitThreads()
    {
        try
        {
            for(int i = 0 ; i < threadCount_ ; i++)
                threads_[i].join();
        }
        catch(InterruptedException e)
        {
        }
    }

    //
    // This is a "worker" thread; it just continually removes objects
    // from the queue and calls the worker's doWork() method, until
    // the queue is deactivated or the doWork() method returns false
    //
    public void
    run()
    {
        Object o;
        boolean done = false;

        while((o = queue_.dequeue()) != null && ! done)
            done = ! worker_.doWork(o);
    }
}
