// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util;

/**
 * BooleanOption is a concrete subclass of AbstractOption that implements
 * the standard on/off option.
 *
 * After the option filter has successfully parsed the options, use
 * <code>isDefined()</code> to determine if this option was specified.
 *
 * @author Mark Spruiell
 */
public class BooleanOption extends AbstractOption
{
    private boolean value_;
    private boolean hasValue_;

    /**
     * Class constructor providing only the character form of the option.
     * The name and property forms are initialized to <code>null</code>.
     *
     * @param ch      the character (short) form of the option
     */
    public
    BooleanOption(char ch)
    {
        this(ch, null, null);
    }

    /**
     * Class constructor providing the string form of the option.
     * The character form is initialized to 0, and the property name
     * is initialized to <code>null</code>.
     *
     * @param name    the string (long) form of the option
     */
    public
    BooleanOption(String name)
    {
        this((char)0, name, null);
    }

    /**
     * Class constructor providing the character and string forms of the
     * option. The property name is initialized to <code>null</code>.  At
     * least one of the forms must be provided.
     *
     * @param ch      the character (short) form of the option, or 0 if
     *                this form is not desired
     * @param name    the string (long) form of the option, or
     *                <code>null</code> if this form is not desired
     */
    public
    BooleanOption(char ch, String name)
    {
        this(ch, name, null);
    }

    /**
     * Class constructor providing the character and string forms and the
     * property name of the option.  At least one of the forms must be
     * provided.
     *
     * @param ch         the character (short) form of the option, or 0 if
     *                   this form is not desired
     * @param name       the string (long) form of the option, or
     *                   <code>null</code> if this form is not desired
     * @param property   the property name of the option, or
     *                   <code>null</code> if not desired
     */
    public
    BooleanOption(char ch, String name, String property)
    {
        super(ch, name, property);
        value_ = false;
        hasValue_ = false;
    }

    /**
     * Class constructor providing the character and string forms, the
     * property name and a default value for the option.  By supplying
     * a default value for the option, the option is considered to
     * be <I>defined</I>.  If the option appears in the argument list
     * or as a property, that value will override the default value.
     * At least one of the names must be non-<code>null</code>.
     *
     * @param ch         the character (short) form of the option, or 0 if
     *                   this form is not desired
     * @param name       the string (long) form of the option, or
     *                   <code>null</code> if this form is not desired
     * @param property   the property name of the option, or
     *                   <code>null</code> if not desired
     * @param defaultValue   the default value
     */
    public
    BooleanOption(char ch, String name, String property, boolean defaultValue)
    {
        super(ch, name, property);
        value_ = defaultValue;
        hasValue_ = true;
    }

    /**
     * Parses a property value.
     *
     * @param value   should be either <code>"true"</code> or
     *                <code>"false"</code>
     *
     * @exception OptionException  If the property value is invalid
     */
    public void
    parseProperty(String value)
        throws OptionException
    {
        if(value.equalsIgnoreCase("true"))
        {
            hasValue_ = true;
            value_ = true;
            parseAction();
        }
        else if(value.equalsIgnoreCase("false"))
        {
            hasValue_ = true;
            value_ = false;
            parseAction();
        }
        else
            throw new OptionException("Invalid value `" + value + "' for " +
                                      "property " + getProperty() +
                                      " - expected true or false");
    }

    /**
     * Parses a command-line argument.
     *
     * @param args     the array of arguments
     * @param offset   the index into <code>args</code> at which we find
     *                 our argument
     *
     * @exception OptionException  If the argument is invalid
     */
    public void
    parse(String[] args, int offset)
        throws OptionException
    {
        args[offset] = null;
        hasValue_ = true;
        value_ = true;
        parseAction();
    }

    /**
     * Determines whether this option was specified.  If this option
     * has a default value, then this method will always return
     * <code>true</code>.
     *
     * @return <code>true</code> if the option was specified, or if
     *         the option has a default value; <code>false</code> otherwise.
     */
    public boolean
    isDefined()
    {
        return hasValue_;
    }

    /**
     * Gets the value of this option.  The caller is responsible for
     * first ensuring that the option was defined by invoking
     * <code>isDefined()</code>.
     *
     * @return the option value
     */
    public boolean
    getValue()
    {
        return value_;
    }

    /**
     * Provides an opportunity for subclassing.  This method is invoked
     * by the <code>parse()</code> method after the option has been
     * successfully parsed, to allow subclasses to perform some action.
     *
     * @exception OptionException  If an error occurred
     */
    protected void
    parseAction()
        throws OptionException
    {
    }
}
