// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util.GUI;

/**
 * GUIOptionFilter is a subclass of OptionFilter that supports some
 * well-known GUI options.  In particular, the following options
 * are defined:
 *
 * <DL>
 *   <DT><B>--look <I>class</I></B></DT>
 *   <DD>Specifies the classname of a JFC Look and Feel class.
 *
 *   <DT><B>--windows</B></DT>
 *   <DD>Requests the use of the Windows Look and Feel class.
 *
 *   <DT><B>--motif</B></DT>
 *   <DD>Requests the use of the Motif Look and Feel class.
 *
 *   <DT><B>--mac</B></DT>
 *   <DD>Requests the use of the Macintosh Look and Feel class.
 * </DL>
 *
 * GUIOptionFilter automatically installs the requested Look and Feel
 * class in the <code>parse</code> method.
 *
 * @author Mark Spruiell
 */
public class GUIOptionFilter extends com.ooc.Util.OptionFilter
{
    private com.ooc.Util.BooleanOption windows_;
    private com.ooc.Util.BooleanOption motif_;
    private com.ooc.Util.BooleanOption mac_;
    private com.ooc.Util.StringOption lookClass_;

    /**
     * Class constructor that installs the standard GUI options.
     */
    public
    GUIOptionFilter()
    {
        windows_ = new com.ooc.Util.BooleanOption("windows");
        motif_ = new com.ooc.Util.BooleanOption("motif");
        mac_ = new com.ooc.Util.BooleanOption("mac");
        lookClass_ = new com.ooc.Util.StringOption("look");

        addOption(windows_);
        addOption(motif_);
        addOption(mac_);
        addOption(lookClass_);
    }

    /**
     * Overridden from <code>OptionFilter</code> to install the requested
     * Look and Feel class.
     */
    public String[]
    parse(String[] args, java.util.Properties props)
        throws com.ooc.Util.OptionException
    {
        //
        // First, let the super class parse the options (and throw
        // any exceptions)
        //
        String[] result = super.parse(args, props);

        //
        // Now determine which (if any) Look & Feel class we should use
        //
        String lookClassName = null;

        if(lookClass_.isDefined())
            lookClassName = lookClass_.getValue();
        else if(windows_.isDefined())
            lookClassName =
                "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
        else if(motif_.isDefined())
            lookClassName =
                "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
        else if(mac_.isDefined())
            lookClassName =
                "com.sun.java.swing.plaf.mac.MacLookAndFeel";

        //
        // Establish the Look & Feel class
        //
        if(lookClassName != null)
        {
            try
            {
                javax.swing.UIManager.setLookAndFeel(lookClassName);
            }
            catch(Exception ex)
            {
		System.err.println(
                    "Unsupported Look & Feel: " + lookClassName);
		System.err.println("Reverting to default Look & Feel");
            }
        }

        return result;
    }
}
