// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util;

/**
 * Option represents the interface to an option object that is expected
 * by the OptionFilter class.  An option can be referred to in one of
 * several ways:
 * <UL>
 *   <LI><I>character</I> - A single-character (short) form
 *   <LI><I>name</I> - A string (long) form
 *   <LI><I>property</I> - A Java property
 * </UL>
 *
 * When specified as a command-line argument, the character form of the
 * option is expected to have a single dash ('-') prefix, whereas the
 * string form is expected to have two dashes.  For example, an option
 * defined using ('h',"help") could be specified as "-h" or "--help".
 *
 * @author Mark Spruiell
 * @see OptionFilter
 */
public interface Option
{
    /**
     * Gets the character used as the short form of this option.
     *
     * @return the character for this option, or 0 if no
     *         character was defined
     */
    public char
    getChar();

    /**
     * Gets the name for this option.
     *
     * @return the name of this option, or <code>null</code> if no
     *         name was defined
     */
    public String
    getName();

    /**
     * Gets the property name for this option.
     *
     * @return the property name of this option, or <code>null</code> if no
     *         property name was defined
     */
    public String
    getProperty();

    /**
     * Parse the property value that was found for this option.
     *
     * @exception OptionException If the property value was invalid
     */
    public void
    parseProperty(String value)
        throws OptionException;

    /**
     * Parse the argument found at index <code>offset</code> in
     * <code>args</code>.  This method is not invoked unless the
     * option filter has found an argument that matches either the
     * character or name for this option.
     * <P>
     * The argument at index <code>offset</code>, as well as any
     * additional parameters consumed by this option, should be
     * set to <code>null</code> upon successful completion
     * of this method.
     *
     * @param args     the argument list
     * @param offset   the index into <code>args</code> at which parsing
     *                 should begin
     *
     * @exception OptionException If an error was detected in the argument list
     */
    public void
    parse(String[] args, int offset)
        throws OptionException;

    /**
     * Produces a human-readable description of this option.
     *
     * @return a description of this option
     */
    public String
    toString();
}
