// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util;

/**
 * OptionFilter handles the details of processing command-line options.
 * Any object that implements the Option interface can be managed by
 * an OptionFilter object.  Option definitions can be obtained from
 * command-line arguments, as well as from Java properties.
 *
 * @author Mark Spruiell
 * @see Option
 */
public class OptionFilter
{
    protected java.util.Hashtable options_ = new java.util.Hashtable();
    protected java.util.Vector vec_ = new java.util.Vector();

    public
    OptionFilter()
    {
    }

    /**
     * Registers a new option object with the filter.
     *
     * @param option   the new option
     */
    public void
    addOption(Option option)
    {
        if(option.getChar() != 0)
            options_.put("-" + option.getChar(), option);

        if(option.getName() != null)
            options_.put("--" + option.getName(), option);

        vec_.addElement(option);
    }

    /**
     * Gets an option by its character (short) form.
     *
     * @param ch   the character form of the option to be found
     *
     * @return the desired option, or <code>null</code> if the option was
     *         not found
     */
    public Option
    getOption(char ch)
    {
        Option result = null;

        java.util.Enumeration e = vec_.elements();
        while(e.hasMoreElements() && result == null)
        {
            Option opt = (Option)e.nextElement();
            if(ch == opt.getChar())
                result = opt;
        }

        return result;
    }

    /**
     * Gets an option by its string (long) form.
     *
     * @param name   the string form of the option to be found
     *
     * @return the desired option, or <code>null</code> if the option was
     *         not found
     */
    public Option
    getOption(String name)
    {
        Option result = null;

        java.util.Enumeration e = vec_.elements();
        while(e.hasMoreElements() && result == null)
        {
            Option opt = (Option)e.nextElement();
            if(name.equals(opt.getName()))
                result = opt;
        }

        return result;
    }

    /**
     * Parses the command-line arguments.  The JVM's system properties
     * will also be searched for matching option definitions.
     *
     * @param args  the program's command-line arguments
     * @return      the remaining command-line arguments, with all
     *              matching options removed
     *
     * @exception OptionException  If an error occurred while processing the
     *                             options
     */
    public String[]
    parse(String[] args)
        throws OptionException
    {
        return parse(args, null);
    }

    /**
     * Searches through the properties and command-line arguments for
     * option definitions.  When a property is found that matches an
     * option's property name, the option's <code>parseProperty()</code>
     * method is invoked.  When a matching command-line argument is
     * found, the option's <code>parse()</code> method is invoked.
     *
     * @param args   the program's command-line arguments
     * @param props  the set of Java properties to be searched, or
     *               <code>null</code> to use the system properties
     * @return       the remaining command-line arguments, with all
     *               matching options removed
     *
     * @exception OptionException  If an error occurred while processing the
     *                             options
     */
    public String[]
    parse(String[] args, java.util.Properties props)
        throws OptionException
    {
        if(props == null)
            props = System.getProperties();

        java.util.Enumeration e = vec_.elements();
        while(e.hasMoreElements())
        {
            Option option = (Option)e.nextElement();
            String propName = option.getProperty();
            if(propName != null)
            {
                String value = props.getProperty(propName);
                if(value != null)
                    option.parseProperty(value);
            }
        }


        java.util.Vector rest = new java.util.Vector();

        for(int i = 0 ; i < args.length ; i++)
        {
            if(args[i] != null)
            {
                if(options_.containsKey(args[i]))
                {
                    Option option = (Option)options_.get(args[i]);
                    option.parse(args, i);
                }
                else
                    rest.addElement(args[i]);
            }
        }

        String[] result = new String[rest.size()];
        rest.copyInto(result);

        return result;
    }

    /**
     * Produces a human-readable summary of the options registered
     * with this option filter.
     *
     * @return  a summary of the registered options
     */
    public String
    toString()
    {
        StringBuffer buff = new StringBuffer();

        java.util.Enumeration e = vec_.elements();
        while(e.hasMoreElements())
        {
            Option option = (Option)e.nextElement();
            buff.append(option.toString() + "\n");
        }

        return buff.toString();
    }
}
