// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util;

/**
 * StringOption is a concrete subclass of AbstractOption that implements
 * an option with a fixed number of String parameters.
 *
 * After the option filter has successfully parsed the options, use
 * <code>isDefined()</code> to determine if this option was specified,
 * and <code>getValue()</code> or <code>getValues()</code> to obtain the
 * option's parameter(s).
 *
 * @author Mark Spruiell
 */
public class StringOption extends AbstractOption
{
    private java.util.Vector values_ = new java.util.Vector();
    int numParams_ = 1;
    private boolean hasValue_ = false;

    /**
     * Class constructor providing only the character form of the option.
     * The name and property forms are initialized to <code>null</code>.
     * This option is expected to have a single parameter, and has no
     * default value.
     *
     * @param ch      the character (short) form of the option
     */
    public
    StringOption(char ch)
    {
        this(ch, null, null);
    }

    /**
     * Class constructor providing the string form of the option.
     * The character form is initialized to 0, and the property name
     * is initialized to <code>null</code>.  This option is expected to
     * have a single parameter, and has no default value.
     *
     * @param name    the string (long) form of the option
     */
    public
    StringOption(String name)
    {
        this((char)0, name, null);
    }

    /**
     * Class constructor providing the character and string forms of the
     * option. The property name is initialized to <code>null</code>.  At
     * least one of the forms must be provided.  This option is expected to
     * have a single parameter, and has no default value.
     *
     * @param ch      the character (short) form of the option, or 0 if
     *                this form is not desired
     * @param name    the string (long) form of the option, or
     *                <code>null</code> if this form is not desired
     */
    public
    StringOption(char ch, String name)
    {
        this(ch, name, null);
    }

    /**
     * Class constructor providing the character and string forms and the
     * property name of the option.  At least one of the forms must be
     * provided.  This option is expected to have a single parameter, and
     * has no default value.
     *
     * @param ch         the character (short) form of the option, or 0 if
     *                   this form is not desired
     * @param name       the string (long) form of the option, or
     *                   <code>null</code> if this form is not desired
     * @param property   the property name of the option, or
     *                   <code>null</code> if not desired
     */
    public
    StringOption(char ch, String name, String property)
    {
        super(ch, name, property);
    }

    /**
     * Class constructor providing the character and string forms and the
     * property name of the option.  At least one of the forms must be
     * provided.  This option is expected to have <code>numParams</code>
     * parameters, and has no default values.
     *
     * @param ch         the character (short) form of the option, or 0 if
     *                   this form is not desired
     * @param name       the string (long) form of the option, or
     *                   <code>null</code> if this form is not desired
     * @param property   the property name of the option, or
     *                   <code>null</code> if not desired
     * @param numParams  the number of parameters expected for the option
     */
    public
    StringOption(char ch, String name, String property, int numParams)
    {
        super(ch, name, property);
        numParams_ = numParams;
    }

    /**
     * Class constructor providing the character and string forms and the
     * property name of the option.  At least one of the forms must be
     * provided.  This option is expected to have a single parameter.
     * By supplying a default value for the option, the option is considered
     * to be <I>defined</I>.  If the option appears in the argument list
     * or as a property, that value will override the default value.
     *
     * @param ch            the character (short) form of the option, or 0 if
     *                      this form is not desired
     * @param name          the string (long) form of the option, or
     *                      <code>null</code> if this form is not desired
     * @param property      the property name of the option, or
     *                      <code>null</code> if not desired
     * @param defaultValue  the default value, or <code>null</code> if
     *                      no default value is desired
     */
    public
    StringOption(char ch, String name, String property, String defaultValue)
    {
        super(ch, name, property);
        hasValue_ = true;
        values_.addElement(defaultValue);
    }

    /**
     * Class constructor providing the character and string forms and the
     * property name of the option.  At least one of the forms must be
     * provided.  This option is expected to have <code>numParams</code>
     * parameters, and has the given default values.
     * By supplying default values for the option, the option is considered
     * to be <I>defined</I>.  If the option appears in the argument list
     * or as a property, those values will override the default values.
     *
     * @param ch             the character (short) form of the option, or 0 if
     *                       this form is not desired
     * @param name           the string (long) form of the option, or
     *                       <code>null</code> if this form is not desired
     * @param property       the property name of the option, or
     *                       <code>null</code> if not desired
     * @param numParams      the number of parameters expected for the option
     * @param defaultValues  the default values, or <code>null</code> if no
     *                       default values are desired; the length of this
     *                       array must match <code>numParams</code>
     */
    public
    StringOption(char ch,
                 String name,
                 String property,
                 int numParams,
                 String[] defaultValues)
    {
        super(ch, name, property);

        if(numParams != defaultValues.length)
            throw new IllegalArgumentException("Length of defaultValues " +
                                               "!= numParams");

        numParams_ = numParams;
        hasValue_ = true;
        for(int i = 0 ; i < defaultValues.length ; i++)
            values_.addElement(defaultValues[i]);
    }

    /**
     * Parses a property value.
     *
     * @param value   should be a comma-separated list of strings;
     *                the list should contain <code>numParams</code>
     *                strings
     *
     * @exception OptionException  If the property value is invalid
     */
    public void
    parseProperty(String value)
        throws OptionException
    {
        java.util.StringTokenizer tokenizer =
            new java.util.StringTokenizer(value, ",");

        if(tokenizer.countTokens() != numParams_)
            throw new OptionException("Invalid value `" + value + "' for " +
                                      "property " + getProperty() +
                                      " - incorrect number of parameters");

        values_.removeAllElements();
        while(tokenizer.hasMoreTokens())
            values_.addElement(tokenizer.nextToken());
    }

    /**
     * Parses a command-line argument.
     *
     * @param args     the array of arguments
     * @param offset   the index into <code>args</code> at which we find
     *                 our argument
     *
     * @exception OptionException  If the arguments are invalid. For
     *   example, if the number of remaining arguments is less than
     *   the number of parameters expected for this option.
     */
    public void
    parse(String[] args, int offset)
        throws OptionException
    {
        if(args.length - (offset + 1) < numParams_)
            throw new OptionException("Expected " + numParams_ +
                                      " argument(s) to option `" +
                                      args[offset] + "'");

        values_ = new java.util.Vector();

        args[offset++] = null;
        for(int i = 0 ; i < numParams_ ; i++)
        {
            values_.addElement(args[offset]);
            args[offset++] = null;
        }

        hasValue_ = true;
        parseAction();
    }

    /**
     * Determines whether this option was specified.  If this option
     * has default values, then this method will always return
     * <code>true</code>.
     *
     * @return <code>true</code> if the option was specified, or if
     *         the option has default values; <code>false</code> otherwise.
     */
    public boolean
    isDefined()
    {
        return hasValue_;
    }

    /**
     * Gets this option's parameters as an array of Strings.
     *
     * @return  the parameter values
     */
    public String[]
    getValues()
    {
        String[] result = new String[values_.size()];
        values_.copyInto(result);
        return result;
    }

    /**
     * Gets this option's first parameter.
     *
     * @return  the first parameter value
     */
    public String
    getValue()
    {
        return (String)values_.elementAt(0);
    }

    /**
     * Provides an opportunity for subclassing.  This method is invoked
     * by the <code>parse()</code> method after the option has been
     * successfully parsed, to allow subclasses to perform some action.
     *
     * @exception OptionException  If an error occurred
     */
    protected void
    parseAction()
        throws OptionException
    {
    }
}
